#include <config.h>

#ifndef NO_STANDALONE

/* Yes, I know, I could have used multi-hashed linked lists, 
 * binary trees and other funny stuff. It'd have been better, cleaner,
 * faster. Yeah, it'd have added many lines of complicated source code
 * to win 1/100000 sec on system with more than 500 sim users.
 * Great win, but sorry, I won't do that just to please theorical
 * algorithms wizards. -Jedi.
 */

# include "ftpd.h"
# include "dynamic.h"
# include "ftpwho-update.h"
# include "globals.h"
#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

static IPTrack *iptrack_list;
static size_t iptrack_size;

static IPTrack *iptrack_find_pid(const pid_t pid)
{
	IPTrack *iptrack_list_pnt;
	size_t iptrack_size_rest = iptrack_size;
	
	if (pid == (pid_t) 0) {
		return NULL;
	}
	iptrack_list_pnt = iptrack_list;
	while (iptrack_size_rest > (size_t) 0U) {
		if (iptrack_list_pnt->pid == pid) {
			return iptrack_list_pnt;
		}
		iptrack_list_pnt++;
		iptrack_size_rest -= sizeof *iptrack_list_pnt;
	}
	return NULL;
}

static IPTrack *iptrack_find_other(const struct sockaddr_storage * const ip)
{
	IPTrack *iptrack_list_pnt;
	size_t iptrack_size_rest = iptrack_size;
	
	if (iptrack_list == NULL) {
		return NULL;
	}
	iptrack_list_pnt = iptrack_list;
	while (iptrack_size_rest > (size_t) 0U) {
		if (STORAGE_FAMILY(iptrack_list_pnt->ip) != STORAGE_FAMILY(*ip)) {
			return iptrack_list_pnt;			
		}		
		if (STORAGE_FAMILY(iptrack_list_pnt->ip) == AF_INET &&
			STORAGE_SIN_ADDR(iptrack_list_pnt->ip) != STORAGE_SIN_ADDR(*ip)) {
			return iptrack_list_pnt;
		}
		if (STORAGE_FAMILY(iptrack_list_pnt->ip) == AF_INET6 &&
			!IN6_ARE_ADDR_EQUAL(&STORAGE_SIN_ADDR6_NF(iptrack_list_pnt->ip),
								&STORAGE_SIN_ADDR6_NF(*ip))) {
			return iptrack_list_pnt;
		}
		iptrack_list_pnt++;
		iptrack_size_rest -= sizeof *iptrack_list_pnt;
	}
	return NULL;
}

static void iptrack_delete_item(IPTrack *iptrack)
{
	ssize_t scrolling;
	IPTrack *new_iptrack_list;

	if (iptrack_list == NULL) {		   /* should never happen */
		return;
	}
	if (iptrack != iptrack_list && iptrack_size > sizeof *iptrack) {
		scrolling = iptrack_size - 
			(((char *) iptrack - (char *) iptrack_list) + sizeof *iptrack);
		if (scrolling > (ssize_t) 0) {
			memmove(iptrack, ((char *) iptrack) + sizeof *iptrack, scrolling);
		}
	}
	if (iptrack == iptrack_list || iptrack_size <= sizeof *iptrack) {
		free(iptrack_list);
		iptrack_list = NULL;
		iptrack_size = (size_t) 0U;
		return;
	}
	iptrack_size -= sizeof *iptrack;
	if ((new_iptrack_list = realloc(iptrack_list, iptrack_size)) != NULL) {
		iptrack_list = new_iptrack_list;
	}
}

void iptrack_delete_pid(const pid_t pid)
{
	IPTrack *iptrack;
	
	if ((iptrack = iptrack_find_pid(pid)) == NULL) {
		return;
	}
	iptrack_delete_item(iptrack);
}

unsigned int iptrack_get(const struct sockaddr_storage * const ip)
{
	IPTrack *iptrack_list_pnt;
	size_t iptrack_size_rest = iptrack_size;
	unsigned int nb = 0U;
	
	if (iptrack_list == NULL) {
		return 0;
	}
	iptrack_list_pnt = iptrack_list;
	while (iptrack_size_rest > (size_t) 0U && nb < UINT_MAX) {
		if (STORAGE_FAMILY(iptrack_list_pnt->ip) != STORAGE_FAMILY(*ip)) {
		} else if (STORAGE_FAMILY(iptrack_list_pnt->ip) == AF_INET &&
			STORAGE_SIN_ADDR(iptrack_list_pnt->ip) == STORAGE_SIN_ADDR(*ip)) {
			nb++;
		} else if (STORAGE_FAMILY(iptrack_list_pnt->ip) == AF_INET6 &&
			IN6_ARE_ADDR_EQUAL(&STORAGE_SIN_ADDR6_NF(iptrack_list_pnt->ip),
							   &STORAGE_SIN_ADDR6_NF(*ip))) {
			nb++;
		}
		iptrack_list_pnt++;
		iptrack_size_rest -= sizeof *iptrack_list_pnt;
	}
	return nb;
}

void iptrack_free(void)
{
	if (iptrack_list != NULL) {
		free(iptrack_list);
		iptrack_list = NULL;
	}
}

void iptrack_add(const struct sockaddr_storage * const ip,
				 const pid_t pid)
{
	IPTrack *iptrack;
		
	if (iptrack_list == NULL) {
		iptrack_size = (size_t) 0U;			
		if ((iptrack_list = malloc(sizeof *iptrack)) == NULL) {
			return;
		}		
	} else {
		IPTrack *new_iptrack_list;
	
		if ((iptrack_size + sizeof *iptrack) > MAX_IPTRACK_SIZE) {
			IPTrack *same;

			if ((same = iptrack_find_other(ip)) != NULL) {
				iptrack_delete_item(same);
			} else {
				iptrack_delete_item(iptrack_list);
			}
		}
		new_iptrack_list = (iptrack_list == NULL ? 
							malloc(iptrack_size + sizeof *iptrack) :
							realloc(iptrack_list, iptrack_size + sizeof *iptrack));
		if (new_iptrack_list == NULL) {
			return;
		}
		iptrack_list = new_iptrack_list;		
	}
	/* Check alignment */
	iptrack = (IPTrack *) (((char *) iptrack_list) + iptrack_size);
	memcpy(&iptrack->ip, ip, sizeof *ip);
	iptrack->pid = pid;
	iptrack_size += sizeof *iptrack;
}

#else
extern signed char v6ready;
#endif
