#ifndef __FTPD_H__
#define __FTPD_H__ 1

#include <stdio.h>
#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
# include <stdarg.h>
#else
# if HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif
#if HAVE_STRING_H
# if !STDC_HEADERS && HAVE_MEMORY_H
#  include <memory.h>
# endif
# include <string.h>
#else
# if HAVE_STRINGS_H
#  include <strings.h>
# endif
#endif
#if HAVE_INTTYPES_H
# include <inttypes.h>
#endif
#include <limits.h>
#include <errno.h>
#include <ctype.h>
#include <signal.h>
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#include <sys/types.h>
#include <sys/stat.h>
#ifdef HAVE_FCNTL_H
# include <fcntl.h>
#elif defined(HAVE_SYS_FCNTL_H)
# include <sys/fcntl.h>
#endif
#ifdef HAVE_IOCTL_H
# include <ioctl.h>
#elif defined(HAVE_SYS_IOCTL_H)
# include <sys/ioctl.h>
#endif
#include <sys/socket.h>
#ifdef HAVE_NETINET_IN_SYSTM_H
# include <netinet/in_systm.h>
#endif
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <sys/mman.h>
#ifdef HAVE_SYS_RESOURCE_H
# include <sys/resource.h>
#endif
#ifdef HAVE_SYS_PARAM_H
# include <sys/param.h>
#endif
#ifdef HAVE_SYS_MOUNT_H
# include <sys/mount.h>
#endif
#include <pwd.h>
#include <grp.h>
#include <arpa/inet.h>
#include <netdb.h>
#ifdef HAVE_SYS_VFS_H
# include <sys/vfs.h>
#endif
#ifdef HAVE_SYS_STATVFS_H
# include <sys/statvfs.h>
#endif
#ifdef HAVE_SYS_LOADAVG_H
# include <sys/loadavg.h>
#endif

/* We can't have both implementations, so if both were
 * found, the configure test failed - Use none */
#if defined(SENDFILE_FREEBSD) && defined(SENDFILE_LINUX)
# undef SENDFILE_FREEBSD
# undef SENDFILE_LINUX
#endif
#ifdef SENDFILE_FREEBSD
/* FreeBSD sendfile() handles large files */
# undef WITH_LARGE_FILES
# define WITH_LARGE_FILES
#elif defined(SENDFILE_LINUX)
# ifdef WITH_LARGE_FILES
#  undef SENDFILE_LINUX
#  define SENDFILE_NONE
# endif
#else
# define SENDFILE_NONE
# undef WITH_LARGE_FILES
# define WITH_LARGE_FILES
#endif

#ifdef HAVE_SYS_UIO_H
# include <sys/uio.h>
#endif
#if defined(SENDFILE_LINUX) && defined(HAVE_SYS_SENDFILE_H)
# include <sys/sendfile.h>
#endif

#ifdef HAVE_SYS_FSUID_H
# include <sys/fsuid.h>
#endif
#ifdef HAVE_ALLOCA
# ifdef HAVE_ALLOCA_H
#  include <alloca.h>
# endif
# define ALLOCA(X) alloca(X)
# define ALLOCA_FREE(X) do { } while (0)
#else
# define ALLOCA(X) malloc(X)
# define ALLOCA_FREE(X) free(X)
#endif

#ifdef __FTPD_P_H__
# define SYSLOG_NAMES	1		/* for -f */
#endif
#include <syslog.h>
#ifndef HAVE_SYSLOG_NAMES
# include "syslognames.h"
#endif

#ifdef USE_SHADOW
# ifdef HAVE_SHADOW_H
#  include <shadow.h>
# endif
#endif

#ifndef O_NOFOLLOW
# define O_NOFOLLOW 0
#endif

#ifndef MAP_FILE
# define MAP_FILE 0
#endif

/*
 * It's a hell. Some OS (Linux, BSD) have statfs.
 * Other OS (Solaris) have statvfs.
 * They do the same thing, but with a different syntax.
 * And Solaris wants statvfs_t, while other only have struct statvfs
 */

#ifdef HAVE_STATVFS
# ifdef HAVE_STATVFS_T
#  define STATFS_STRUCT statvfs_t
# else
#  define STATFS_STRUCT struct statvfs
# endif
# define STATFS(PATH, STR) statvfs(PATH, STR)
# define FSTATFS(FD, STR) fstatvfs(FD, STR)
# define STATFS_BAVAIL(X) ((X).f_bavail)
# define STATFS_BLOCKS(X) ((X).f_blocks)
# define STATFS_BSIZE(X) ((X).f_bsize)
# define STATFS_FRSIZE(X) ((X).f_frsize)
#elif defined(HAVE_STATFS)
# define STATFS_STRUCT struct statfs
# define STATFS(PATH, STR) statfs(PATH, STR)
# define FSTATFS(FD, STR) fstatfs(FD, STR)
# define STATFS_BAVAIL(X) ((X).f_bavail)
# define STATFS_BLOCKS(X) ((X).f_blocks)
# define STATFS_FRSIZE(X) ((X).f_bsize)
# define STATFS_BSIZE(X) ((X).f_bsize)
#else
# define STATFS_STRUCT int
# define STATFS(PATH, STR) (-1)
# define FSTATFS(FD, STR) (-1)
# define STATFS_BAVAIL(X) (0)
# define STATFS_BLOCKS(X) (0)
# define STATFS_BSIZE(X) (0)
# define STATFS_FRSIZE(X) (0)
#endif

#ifndef errno
extern int errno;
#endif

typedef struct FileInfo_ {
	char **names_pnt;
	size_t name_offset;
	off_t size;
	time_t mtime;
	mode_t mode;
	nlink_t nlink;
	uid_t uid;
	gid_t gid;
} FileInfo;

#define FI_NAME(X) (*((X)->names_pnt) + (X)->name_offset)

void *aborttransfer(int);
void parser(void);
void dobanner(void);
void douser(const char *name);
void dopass(const char *password);
void docwd(const char *dir);
void doretr(char *name);
void dorest(const char *name);
void dodele(char *name);
void dostor(char *name, const int append, const int autorename);
void domkd(char *name);
void dormd(char *name);
void domdtm(const char *name);
void dosize(const char *name);
void doeprt(char *p);
void doport(const char *arg);
void doport2(struct sockaddr_storage a, unsigned int p);
void dopasv(int);
void dochmod(char *name, const mode_t mode);
void error(int n, const char *msg);
void domode(const char *arg);
void dostru(const char *arg);
void dotype(const char *arg);
void donoop(void);
void dornfr(char *name);
void dornto(char *name);
void dostou(void);
void dofeat(void);
void domlst(const char * const file);
void mappedtov4(struct sockaddr_storage *ss);

void getnames(void);
void donlist(char *arg, int on_ctrlconn);
int opendata(void);
void addreply(int, const char *, ...);
void addreply_noformat(int, const char *);
void doreply(void);
void sighandler(int sig);
void prevent(char *arg);
int daemons(unsigned short server_port);
void logfile(const int facility, const char *format, ...);
void die(const int err, const int facility, const char * const format, ...);
void die_mem(void);
void setprogname(const char * const title);
int modernformat(const char *file,
				 char *target, size_t target_size);
struct passwd *getpwnam_unix(const char * const name);
int sfgets(void);
const char *getgroup(const gid_t gid);

#ifdef HAVE_SYS_FSUID_H
# define usleep2 usleep
#else
void usleep2(const unsigned long microsec);
#endif

extern int opt_a, opt_C, opt_d, opt_F, opt_l, opt_R;

#ifndef VHOST_PATH
# define VHOST_PATH "/etc/pure-ftpd"
#endif

#ifndef PID_FILE
# define PID_FILE "/var/run/pure-ftpd.pid"
#endif

#ifndef NON_ROOT_FTP
# define DEFAULT_FTP_PORT_S "21"
#else
# define DEFAULT_FTP_PORT_S "2121"
#endif

#define DEFAULT_MAX_USERS 50
#define DEFAULT_FTP_DATA_PORT 20 
#define MAX_SYSLOG_LINE (MAXPATHLEN + 512U)
#define DEFAULT_IDLE (15UL * 60UL)
#define MAX_SITE_IDLE (42UL * 60UL)
#define DEFAULT_MAX_LS_FILES 2000U
#define DEFAULT_MAX_LS_DEPTH 5U
#define GLOB_TIMEOUT 17				   /* Max user time for a 'ls' to complete */
#define MAX_CPU_TIME (17 * 60)		   /* Max allowed CPU time per session */

#ifdef LOG_FTP
# define DEFAULT_FACILITY LOG_FTP
#else
# define DEFAULT_FACILITY LOG_LOCAL2
#endif

#ifdef WITH_LDAP
# define MAX_DATA_SIZE (12 * 1024 * 1024)	   /* Max memory usage - LDAP needs more */
#else
# define MAX_DATA_SIZE (8 * 1024 * 1024)	   /* Max memory usage */
#endif

#define MAX_THROTTLING_DELAY 42		   /* Maximum throttling compensation */

#ifdef TCP_CORK
# define CORK_ON(SK) do { int optval = 1; setsockopt(SK, SOL_TCP, TCP_CORK, \
  &optval, sizeof optval); } while(0)
# define CORK_OFF(SK) do { int optval = 0; setsockopt(SK, SOL_TCP, TCP_CORK, \
  &optval, sizeof optval); } while(0)
#else
# define CORK_ON(SK) do { } while(0)
# define CORK_OFF(SK) do { } while(0)
#endif

#define STORAGE_PORT(X) (((struct sockaddr_in *) &(X))->sin_port)
#define STORAGE_PORT6(X) (((struct sockaddr_in6 *) &(X))->sin6_port)
#define STORAGE_SIN_ADDR(X) ((((struct sockaddr_in *) &(X))->sin_addr).s_addr)
#define STORAGE_SIN_ADDR6(X) ((((struct sockaddr_in6 *) &(X))->sin6_addr).s6_addr)
#define STORAGE_SIN_ADDR6_NF(X) (((struct sockaddr_in6 *) &(X))->sin6_addr)

#ifdef HAVE_SS_LEN
# define STORAGE_LEN(X) ((X).ss_len)
# define SET_STORAGE_LEN(X, Y) do { STORAGE_LEN(X) = (Y); } while(0)
#elif defined(HAVE___SS_LEN)
# define STORAGE_LEN(X) ((X).__ss_len)
# define SET_STORAGE_LEN(X, Y) do { STORAGE_LEN(X) = (Y); } while(0)
#else
# define STORAGE_LEN(X) (STORAGE_FAMILY(X) == AF_INET ? sizeof(struct sockaddr_in) : sizeof(struct sockaddr_in6))
# define SET_STORAGE_LEN(X, Y) (void) 0
#endif

#ifdef HAVE___SS_FAMILY
# define STORAGE_FAMILY(X) ((X).__ss_family)
#else
# define STORAGE_FAMILY(X) ((X).ss_family)
#endif

#ifndef SOL_IP
# define SOL_IP IPPROTO_IP
#endif
#ifndef SOL_TCP
# define SOL_TCP IPPROTO_TCP
#endif

#ifndef INADDR_NONE
# define INADDR_NONE 0
#endif

#if !defined(O_NDELAY) && defined(O_NONBLOCK)
# define O_NDELAY O_NONBLOCK
#endif

#ifndef FNDELAY
# define FNDELAY O_NDELAY
#endif

#ifndef MAP_FAILED
# define MAP_FAILED ((void *) -1)
#endif

#define CRLF "\r\n"

#ifdef WITH_DMALLOC
# define _exit(X) exit(X)
#endif

#ifdef FTPWHO
# define _EXIT(X) ftpwho_exit(X)
#else
# define _EXIT(X) _exit(X)
#endif

#endif
