#ifndef __FTPD_P_H__
#define __FTPD_P_H__ 1

#include "bsd-glob.h"

#ifdef USE_PAM
#include "log_pam.h"
#endif

#include "ftpd.h"
#ifdef WITH_LDAP
# include "log_ldap.h"
# include "parser.h"
#endif
#if !defined(NO_GETOPT_LONG) && !defined(HAVE_GETOPT_LONG)
# include "gnu-getopt.h"
#else
# ifdef HAVE_GETOPT_H
#  include <getopt.h>
# endif
#endif

#ifdef HAVE_SYS_WAIT_H
# include <sys/wait.h>
#endif
#ifndef WEXITSTATUS
# define WEXITSTATUS(st) ((unsigned) (st) >> 8)
#endif
#ifndef WIFEXITED
# define WIFEXITED(st) (((st) & 0xff) == 0)
#endif

#define MAP_SIZE (1<<18)

#define DEFAULT_BACKLOG 42
#define DEFAULT_BACKLOG_DATA 8
#define NICE_VALUE 10
#define THROTTLING_UNIT 10000UL
#define MAX_GROUPS 256
#define MAX_PASSWD_TRIES 5		/* Abort after 5 authentication failures */
#define PASSWD_FAILURE_DELAY (2UL*1000000UL)	/* Delay after each failure */

#ifdef WITH_LDAP
struct passwd * (*getpwnam_wrapper) (const char *name) = getpwnam_unix;
# define getgroups(X, Y) (0)
# define initgroups(X, Y) (0)
#else
# define getpwnam_wrapper(X) getpwnam_unix(X)
#endif

#ifndef NO_STANDALONE
static volatile sig_atomic_t stop_server;
static const char *standalone_port = DEFAULT_FTP_PORT_S;
static const char *standalone_ip;
static volatile unsigned int nb_children;
static int listenfd = -1;
#endif

#ifndef NO_INETD
static signed char standalone;
#endif

#ifdef WITH_LDAP
static char *ldap_server;
static char *ldap_port_s;
static int ldap_port;
static char *ldap_base_dn;
static char *ldap_bind_dn;
static char *ldap_bind_pw;

static ConfigKeywords ldap_config_keywords[] = {
	{ "LDAPServer", &ldap_server },
	{ "LDAPPort", &ldap_port_s },	
	{ "LDAPBaseDN", &ldap_base_dn },
	{ "LDAPBindDN", &ldap_bind_dn },		
	{ "LDAPBindPW", &ldap_bind_pw },
	{ NULL, NULL }
};
#endif

struct reply {
	struct reply *prev;
	char line[1];
};

static const char *GETOPT_OPTIONS =
	"4Aa:bc:"
#ifndef NO_STANDALONE
	"BC:"
#endif
	"dDeEf:"
#ifdef COOKIE
	"F:"
#endif
#ifndef NO_GETOPT_LONG	
	"h"
#endif
	"HiI:k:"
#ifdef WITH_LDAP
	"l:"
#endif
	"L:Mm:N"
#if defined(WITH_UPLOAD_SCRIPT) && !defined(NON_ROOT_FTP)
	"o"
#endif
	"p:P:r"
#ifdef RATIOS
	"q:Q:"
#endif
	"s"
#ifndef NO_STANDALONE
	"S:"
#endif
#ifdef THROTTLING
	"t:T:"
#endif
	"u:U:"
#ifdef VUSERS
	"v:"
#endif
	"V:wWxXz";

#ifndef NO_GETOPT_LONG	
static struct option long_options[] = {
	{ "ipv4only", 0, NULL, '4' },
	{ "chrooteveryone", 0, NULL, 'A' },
	{ "trustedgid", 1, NULL, 'a' },
	{ "brokenclientscompatibility", 0, NULL, 'b' },
	{ "maxclientsnumber", 1, NULL, 'c' },
# ifndef NO_STANDALONE
	{ "daemonize", 0, NULL, 'B' },				
	{ "maxclientsperip", 1, NULL, 'C' },
# endif
	{ "verboselog", 0, NULL, 'd' },
	{ "displaydotfiles", 0, NULL, 'D' },
	{ "anonymousonly", 0, NULL, 'e' },
	{ "noanonymous", 0, NULL, 'E' },
	{ "syslogfacility", 1, NULL, 'f' },
# ifdef COOKIE
	{ "fortunesfile", 1, NULL, 'F' },
# endif
	{ "help", 0, NULL, 'h' },		
	{ "dontresolve", 0, NULL, 'H' },
	{ "maxidletime", 1, NULL, 'I' },
	{ "anonymouscantupload", 0, NULL, 'i' },
	{ "maxdiskusagepct", 1, NULL, 'k' },
# ifdef WITH_LDAP
	{ "ldapconfigfile", 1, NULL, 'l' },
# endif
	{ "limitrecursion", 1, NULL, 'L' },
	{ "anonymouscancreatedirs", 0, NULL, 'M' },
	{ "maxload", 1, NULL, 'm' },
	{ "natmode", 0, NULL, 'N' },
# if defined(WITH_UPLOAD_SCRIPT) && !defined(NON_ROOT_FTP)
	{ "uploadscript", 0, NULL, 'o' },
# endif
	{ "passiveportrange", 1, NULL, 'p' },
	{ "forcepassiveip", 1, NULL, 'P' },
# ifdef RATIOS
	{ "anonymousratio", 1, NULL, 'q' },
	{ "userratio", 1, NULL, 'Q' },
# endif
	{ "autorename", 0, NULL, 'r' },
	{ "antiwarez", 0, NULL, 's' },
# ifndef NO_STANDALONE
	{ "bind", 1, NULL, 'S' },
# endif
	{ "anonymousbandwidth", 1, NULL, 't' },
	{ "userbandwidth", 1, NULL, 'T' },
	{ "umask", 1, NULL, 'U' },
	{ "minuid", 1, NULL, 'u' },
# ifdef VUSERS
	{ "virtualusers", 1, NULL, 'v' },
# endif
	{ "trustedip", 1, NULL, 'V' },
	{ "allowuserfxp", 0, NULL, 'w' },
	{ "allowanonymousfxp", 0, NULL, 'W' },
	{ "prohibitdotfileswrite", 0, NULL, 'x' },
	{ "prohibitdotfilesread", 0, NULL, 'X' },
	{ "allowdotfiles", 0, NULL, 'z' },		
	{ NULL, 0, NULL, 0 }
};

#endif

#ifndef HAVE_SYS_FSUID_H
static sigset_t old_sigmask;
#endif

#endif
	
