#include <config.h>

#ifdef FTPWHO
# include "ftpd.h"
# include "ftpwho-update_p.h"
# include "ftpwho-update.h"
# include "globals.h"
# ifdef WITH_DMALLOC
#  include <dmalloc.h>
# endif

void ftpwho_exit(const int ret)
{
	if (shm_data_cur != NULL) {
		/* Do we really need a lock for this ? */
		shm_data_cur->state = FTPWHO_STATE_FREE;
	}
	if (shm_data != NULL) {
		shmdt((void *) shm_data);
	}
	_exit(ret);
}

void ftpwho_unlock(void)
{
	struct sembuf sb;
	
	if (sem_id < 0 || sem_nb < 0) {
		return;
	}
	sb.sem_num = sem_nb;
	sb.sem_op = 1;
	sb.sem_flg = SEM_UNDO;
	semop(sem_id, &sb, 1);	
}

void ftpwho_lock(void)
{
	struct sembuf sb;
	
	if (sem_id < 0 || sem_nb < 0) {
		return;
	}
	/* Are we sure there are no race condition here ? */
	sb.sem_num = sem_nb;
	sb.sem_op = -1;
	sb.sem_flg = SEM_UNDO;
	semop(sem_id, &sb, 1);
}

int ftpwho_initwho(const int create, size_t * const seg_size)
{
	int fight = 42;
	FTPWhoEntry *scanned_entry;
	FTPWhoEntry *scanned_entry_end;
	unsigned int maxusers_shm;

	/* First, the shared segment */
	*seg_size = (size_t) 0U;
	if (maxusers > 0U) {
		maxusers_shm = maxusers;
	} else {
		maxusers_shm = DEFAULT_MAX_USERS;
	}
	tryagain:
	if ((shm_id = shmget(FTPWHO_KEY, 0, 0)) < 0) {
		/* the shared segment doesn't exist, create it */
		shm_id = shmget(FTPWHO_KEY, (int) (maxusers_shm * sizeof(FTPWhoEntry)),
						IPC_CREAT | IPC_EXCL | 0600);
		if (shm_id < 0) {			   /* stolen ? */
			war:
			if (--fight > 0) {
				goto tryagain;
			}
			logfile(LOG_WARNING, "shctl war : %s", strerror(errno));
			return -1;
		}
		if ((shm_data = shmat(shm_id, NULL, 0)) == NULL) {
			logfile(LOG_WARNING, "shmat : %s", strerror(errno));
			return -1;
		}
		memset(shm_data, 0, maxusers_shm * sizeof(FTPWhoEntry));
	} else {
		struct shmid_ds shm_stat;
		
		if (shmctl(shm_id, IPC_STAT, &shm_stat) < 0) {
			goto war;
		}
		if (
#ifndef NON_ROOT_FTP
			shm_stat.shm_perm.uid != (uid_t) 0 ||
			shm_stat.shm_perm.cuid != (uid_t) 0 ||
#endif
			(*seg_size = shm_stat.shm_segsz) < 
			maxusers_shm * sizeof(FTPWhoEntry) ||
			(shm_stat.shm_segsz % sizeof(FTPWhoEntry)) != 0 ||
			(shm_stat.shm_perm.mode & 0777) != 0600) {   /* wrong perms : be paranoid */
			shmctl(shm_id, IPC_RMID, NULL);
			goto war;
		}
		if ((shm_data = shmat(shm_id, NULL, 0)) == NULL) {
			logfile(LOG_WARNING, "shmat : %s", strerror(errno));
			return -1;
		}
	}
	
	/* Now, the semaphores */
	fight = 42;
	tryagain2:
	if ((sem_id = semget(FTPWHO_KEY, 0, 0)) < 0) {
		struct semid_ds sem_stat;
		union semun sem_union;
		
		/* the semaphore doesn't exist, create it */		
		sem_id = semget(FTPWHO_KEY, (int) maxusers_shm, 
						IPC_CREAT | IPC_EXCL | 0600);
		if (sem_id < 0) {
			war2:
			if (--fight > 0) {
				goto tryagain2;
			}
			logfile(LOG_WARNING, "semget war : %s", strerror(errno));
			return -1;
		}
		sem_union.buf = &sem_stat;
		if (semctl(sem_id, 0, IPC_STAT, sem_union) < 0) {
			goto war2;
		}
	} else {
		struct semid_ds sem_stat;
		union semun sem_union;
		
		sem_union.buf = &sem_stat;
		if (semctl(sem_id, 0, IPC_STAT, sem_union) < 0) {
			goto war2;
		}
		if (
#ifndef NON_ROOT_FTP
			sem_stat.sem_perm.uid != (uid_t) 0 ||
			sem_stat.sem_perm.cuid != (uid_t) 0 ||
#endif
			sem_stat.sem_nsems < maxusers ||
			(sem_stat.sem_perm.mode & 0777) != 0600) {   /* wrong perms : be paranoid */
			semctl(sem_id, 0, IPC_RMID, 0);
			goto war2;
		}
	}
	if (sem_id >= 0) {
		int nb = maxusers;		
		union semun sem_union;
		
		sem_union.val = 1;
		do {
			nb--;
			semctl(sem_id, nb, SETVAL, sem_union);
		} while (nb >= 0);
	}
	if (create == 0) {
		return 0;
	}
	/* Okay, now, find a free slot */
	scanned_entry = shm_data;
	scanned_entry_end = &shm_data[maxusers_shm];
	sem_nb = 0;
	do {
		ftpwho_lock();
		if (scanned_entry->state == FTPWHO_STATE_FREE) {
			shm_data_cur = scanned_entry;
			shm_data_cur->state = FTPWHO_STATE_IDLE;
			ftpwho_unlock();
			return 0;
		}
		ftpwho_unlock();
		sem_nb++;
		scanned_entry++;
	} while (scanned_entry != scanned_entry_end);
	sem_nb = -1;

	shm_data_cur = NULL;
	
	return -2;
}
#else
extern signed char v6ready;
#endif
