/*
 * Changed qmail-pop3 to pure-ftpd ...
 * 
 * Program:	Pluggable Authentication Modules login services
 *
 * Author:	Michael K. Johnson
 *		Red Hat Software
 *		Internet: johnsonm@redhat.com
 *
 *
 *  This majority of this code was lifted from the src.rpm for imap
 *  in the RedHat-4.2 updates directory
 *  by Kelley Lingerfelt redhat@cococo.net
 */

#include <config.h>
#ifdef USE_PAM
# include "ftpd.h"
# ifdef HAVE_SECURITY_PAM_MISC_H
#  include <security/pam_misc.h>
# endif
# ifdef HAVE_SECURITY_PAM_APPL_H
#  include <security/pam_appl.h>
# endif
# include <security/pam_modules.h>
# ifdef HAVE_SECURITY_PAM_FILTER_H
#  include <security/pam_filter.h>
# endif
# ifdef WITH_DMALLOC
#  include <dmalloc.h>
# endif

/* Static variables used to communicate between the conversation function
 * and the server_login function
 */
static const char *PAM_username;
static const char *PAM_password;
static int PAM_error = 0;

/* for compability with older pam stuff, before the stupid transposition */
#ifndef PAM_CRED_ESTABLISH
#define PAM_CRED_ESTABLISH  0x0002U
#endif

/* PAM conversation function
 * Here we assume (for now, at least) that echo on means login name, and
 * echo off means password.
 */
static int PAM_conv(int num_msg,
					const struct pam_message **msg,
					struct pam_response **resp, void *appdata_ptr)
{
	int count = 0, replies = 0;
	struct pam_response *reply = NULL;
	int size = sizeof(struct pam_response);
	appdata_ptr = appdata_ptr;

#define GET_MEM if (reply) realloc(reply, size); else reply = malloc(size); \
  if (!reply) return PAM_CONV_ERR; \
  size += sizeof(struct pam_response)
#define COPY_STRING(s) (s) ? strdup(s) : (char *)NULL

	for (count = 0; count < num_msg; count++) {
		switch (msg[count]->msg_style) {
		case PAM_PROMPT_ECHO_ON:
			GET_MEM;
			reply[replies].resp_retcode = PAM_SUCCESS;
			reply[replies++].resp = COPY_STRING(PAM_username);
			/* PAM frees resp */
			break;
		case PAM_PROMPT_ECHO_OFF:
			GET_MEM;
			reply[replies].resp_retcode = PAM_SUCCESS;
			reply[replies++].resp = COPY_STRING(PAM_password);
			/* PAM frees resp */
			break;
		case PAM_TEXT_INFO:
			/* ignore it... */
			break;
		case PAM_ERROR_MSG:
		default:
			/* Must be an error of some sort... */
			free(reply);
			PAM_error = 1;
			return PAM_CONV_ERR;
		}
	}
	if (reply)
		*resp = reply;
	return PAM_SUCCESS;
}
static struct pam_conv PAM_conversation = {
	&PAM_conv,
	NULL
};

/* Server log in
 * Accepts: user name string
 *	    password string
 * Returns: T if password validated, NIL otherwise
 */
int pam_ftp_check(const char *user, const char *pass)
{
	pam_handle_t *pamh;
	int pam_error;
/*  char tmp[MAILTMPLEN];  */
	/* struct passwd *pw = getpwnam (user); */

	/* Now use PAM to do authentication.  For now, we won't worry about
	 * session logging, only authentication.  Bail out if there are any
	 * errors.  Since this is a limited protocol, and an even more limited
	 * function within a server speaking this protocol, we can't be as
	 * verbose as would otherwise make sense.  It would be nice if we
	 * could return a string for the server to pass the the client and/or
	 * log, but that doesn't exist right now.  If it is ever added, PAM
	 * could make good use of it.
	 * Query: should we be using PAM_SILENT to shut PAM up?
	 */
#define PAM_BAIL if (PAM_error || (pam_error != PAM_SUCCESS)) { \
     pam_end(pamh, 0); return 0; \
   }
	PAM_password = pass;
	PAM_username = user;
	pam_error = pam_start("pure-ftpd", user, &PAM_conversation, &pamh);
	PAM_BAIL;
	pam_error = pam_authenticate(pamh, 0);
	PAM_BAIL;
	pam_error = pam_acct_mgmt(pamh, 0);
	PAM_BAIL;
	pam_error = pam_setcred(pamh, PAM_CRED_ESTABLISH);
	/* If this point is reached, the user has been authenticated. */	
	PAM_BAIL;
	/*
	 * Handle session entries. PAM is instructed to shut up for now.
	 * 20010530 <tossu@cc.hut.fi>
	 */
#ifndef WITHOUT_PAM_SESSION
	(void) pam_open_session(pamh, PAM_SILENT);
	(void) pam_close_session(pamh, PAM_SILENT);   /* It doesn't matter if it fails */
#endif
	(void) pam_end(pamh, PAM_SUCCESS);
	return 1;
}
#else
extern signed char v6ready;
#endif
