#include <config.h>

#ifndef WITH_UPLOAD_SCRIPT
#include <stdio.h>

int main(void)
{
	puts("Please compile the server with --with-uploadscript\n"
		 "to use this feature. Thank you.");
	
	return 0;
}
#else

# include "ftpd.h"
# include "upload-pipe.h"
# include "pure-uploadscript_p.h"

static inline void setcloexec(const int fd)
{
	fcntl(fd, F_SETFD, FD_CLOEXEC);
}

static int upload_pipe_ropen(void)
{
	struct stat st;
	int upload_pipe_fd;
	
	if ((upload_pipe_fd =
		 open(UPLOAD_PIPE_FILE, O_RDWR | O_NOFOLLOW)) < 0) {
		perror("Unable to open " UPLOAD_PIPE_FILE);
		return -1;
	}
	setcloexec(upload_pipe_fd);
	if (fstat(upload_pipe_fd, &st) < 0 ||
		(st.st_mode & 0777) != 0600 || !S_ISFIFO(st.st_mode) ||
		st.st_uid != (uid_t) 0) {
		fprintf(stderr, "Insecure permissions on " UPLOAD_PIPE_FILE "\n");
		return -1;
	}
	return upload_pipe_fd;
}

static int readchar(const int upload_file_fd)
{
	ssize_t ret;
	unsigned char c;
	
	while ((ret = read(upload_file_fd, &c, (size_t) 1U)) < (ssize_t) 0 &&
		   errno == EINTR);
	if (ret <= (ssize_t) 0) {
		return EOF;
	} 
	return (int) c;
}

static char *readfile(const int upload_file_fd)
{
	static char buf[MAXPATHLEN + 1U];
	const char * const bufend = &buf[sizeof buf];
	char *bufpnt = buf;
	int c;
	
	do {
		c = readchar(upload_file_fd);
		if (c == EOF) {
			return NULL;
		}
	} while (c != 1);
	while (bufpnt != bufend) {
		c = readchar(upload_file_fd);
		if (c == EOF || (c < 32 && c != 0)) {
			return NULL;
		}
		*bufpnt = c;
		if (c == 0) {
			return buf;
		}
		bufpnt++;
	}
	return NULL;
}

/* 
 * When we are using virtual hosts, the file looks like :
 * <ip address>:<path>
 */

static char *checkvirtual(char *path)
{
	static char buf[MAXPATHLEN + 1U];
	char *path_pnt;
	
	if (path == NULL || *path == '/' ||
		(path_pnt = strstr(path, ":/")) == NULL) {
		return path;
	}
	*path_pnt = 0;
	if (snprintf(buf, sizeof buf, VHOST_PATH "/%s%s", 
				 path, path_pnt + 1) < 0) {
		/* Better avoid processing than risking a security flaw */
		return NULL;
	}
	
	return buf;
}

static void dodaemonize(void)
{
	pid_t child;
	
	if (daemonize) {
		if ((child = fork()) < (pid_t) 0) {
			perror("Daemonization failed - fork");
			return;
		} else if (child != (pid_t) 0) {
			_exit(EXIT_SUCCESS);
		} else {
			if (setsid() == (pid_t) -1) {
			   perror("Daemonization failed : setsid");
			}
			chdir("/");
			if (isatty(1)) {
				close(1);
			}
			if (isatty(2)) {
				close(2);
			}
		}
	}	
}

static int init(void)
{
	close(0);	
	if (!isatty(1) && close(1) < 0) {
		return -1;
	}
	if (!isatty(2) && close(2) < 0) {
		return -1;
	}
	if (geteuid() != (uid_t) 0) {
		fprintf(stderr, "Sorry, but you have to be r00t to run this program\n");
		return -1;
	}
	
	return 0;
}

static void usage(void)
{
#ifndef NO_GETOPT_LONG	
	const struct option *options = long_options;
	
	do {
		printf("-%c\t--%s\t%s\n", options->val, options->name,
			   options->has_arg ? "<opt>" : "");
		options++;
	} while (options->name != NULL);
#endif
	exit(EXIT_SUCCESS);
}

static int parseoptions(int argc, char *argv[])
{
#ifndef NO_GETOPT_LONG
	int option_index = 0;
#endif
	int fodder;

	while ((fodder =
#ifndef NO_GETOPT_LONG
			getopt_long(argc, argv, GETOPT_OPTIONS, long_options, &option_index)
#else
			getopt(argc, argv, GETOPT_OPTIONS)
#endif
			) != -1) {
		switch (fodder) {
		case 'B': {
			daemonize = 1;
			break;
		}
		case 'g': {
			char *nptr, *endptr;
			
			nptr = optarg;
			endptr = NULL;
			gid = (gid_t) strtoul(nptr, &endptr, 10);
			if (!nptr || !*nptr || !endptr || *endptr) {
				perror("Illegal GID - Must be a number\n");
			}
			break;			
		}
#ifndef NO_GETOPT_LONG
		case 'h': {
			usage();
		}
#endif
		case 'r': {
			if ((script = strdup(optarg)) == NULL) {
				perror("Oh no ! More memory !");
			}
			break;			
		}
		case 'u': {
			char *nptr, *endptr;
			
			nptr = optarg;
			endptr = NULL;
			uid = (uid_t) strtoul(nptr, &endptr, 10);
			if (!nptr || !*nptr || !endptr || *endptr) {
				perror("Illegal UID - Must be a number\n");
			}
			break;
		}
		default: 
			usage();
		}
	}
	return 0;
}

static int changeuidgid(void)
{
	if (setgroups(1U, &gid) ||
		setgid(gid) || setegid(gid) ||
		setuid(uid) || seteuid(uid) || chdir("/")) {
		return -1;
	}
	return 0;
}

#ifdef HAVE_PUTENV
static void newenv_ull(const char * const var, const unsigned long long val)
{
    size_t s;
    char *v;
    
    s = strlen(var) + (size_t) 42U;
    if ((v = malloc(s)) == NULL) {
        return;
    }
    if (snprintf(v, s, "%s=%llu", var, val) < 0) {
        free(v);
        return;
    }
    putenv(v);
}

static void newenv_uo(const char * const var, const unsigned int val)
{
    size_t s;
    char *v;
    
    s = strlen(var) + (size_t) 8U;
    if ((v = malloc(s)) == NULL) {
        return;
    }
    if (snprintf(v, s, "%s=%o", var, val) < 0) {
        free(v);
        return;
    }
    putenv(v);
}

static void newenv_str(const char * const var, const char * const str)
{
    size_t s;
    char *v;
    
    if (str == NULL || *str == 0) {
        return;
    }
    s = strlen(var) + strlen(str) + (size_t) 2U;
    if ((v = malloc(s)) == NULL) {
        return;
    }
    if (snprintf(v, s, "%s=%s", var, str) < 0) {
        free(v);
        return;
    }
    putenv(v);
}
#endif

static void fillenv(const struct stat * const st)
{
#ifdef HAVE_PUTENV    
    struct passwd *pwd;
    struct group *grp;
    
    pwd = getpwuid(st->st_uid);
    grp = getgrgid(st->st_gid);
    newenv_ull("UPLOAD_SIZE", (unsigned long long) st->st_size);
    newenv_uo("UPLOAD_PERMS", (unsigned int) (st->st_mode & 07777));
    newenv_ull("UPLOAD_UID", (unsigned long long) st->st_uid);
    newenv_ull("UPLOAD_GID", (unsigned long long) st->st_gid);
    if (pwd != NULL) {
        newenv_str("UPLOAD_USER", pwd->pw_name);
    }
    if (grp != NULL) {
        newenv_str("UPLOAD_GROUP", grp->gr_name);
    }
#else
    (void) st;
#endif
}

static int run(const char * const file, const int upload_pipe_fd)
{
	struct stat st;
	pid_t pid;
	
	if (script == NULL || *script == 0 ||
		file == NULL || *file == 0 ||
		lstat(file, &st) < 0 ||
		!S_ISREG(st.st_mode)) {
		return -1;
	}
	pid = fork();
	if (pid == (pid_t) 0) {
		/* Yes, there's already the cloexec flag on this fd,
		 * but it's really important to close it. Be paranoid.
		 */
		if (close(upload_pipe_fd) < 0) {
			_exit(EXIT_FAILURE);
		}
		close(1);
		close(2);
        fillenv(&st);
		execl(script, script, file, NULL);
		_exit(EXIT_FAILURE);
	} else if (pid != (pid_t) -1) {
#ifdef HAVE_WAITPID
		(void) waitpid(pid, NULL, 0);
#else
		while (wait3(NULL, 0, NULL) != pid);
#endif
	}
	
	return 0;
}

int main(int argc, char *argv[])
{
	int upload_pipe_fd;
	char *file;
	
	(void) argc;
	(void) argv;
	
	if (init() < 0) {
		return -1;
	}
	if (parseoptions(argc, argv) < 0) {
		return -1;
	}
	if (script == NULL) {
		fprintf(stderr, "Sorry, but I need -r <program to run>\n\n");
		usage();
	}
	if (daemonize != 0) {
		dodaemonize();
	}
	if ((upload_pipe_fd = upload_pipe_ropen()) < 0) {
		return -1;
	}
	if (changeuidgid() < 0) {
		perror("Identity change");
		return -1;
	}
	for (;;) {
		file = checkvirtual(readfile(upload_pipe_fd));
		if (file != NULL) {
			run(file, upload_pipe_fd);
		}
	}
	close(upload_pipe_fd);
	
	return 0;
}

#endif


